using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace Phasma
{
    /// <summary>
    /// RLMenu that functions like an input box, with an appearance to fit an all-ascii gui.
    /// </summary>
    public class RLInput : RLMenu
    {
        /// <summary>
        /// Pops up an instance of RLInput and prompts the user for a string.
        /// </summary>
        /// <param name="myDisplay">RLDisplay object to associate this window with.</param>
        /// <param name="myKeybinder">RLKeybindController object that normall handles input.</param>
        /// <param name="Prompt">Prompt string to be shown to the user.</param>
        /// <returns>The user's input string. Empty if the user hits escape.</returns>
        public static string PopInput(RLDisplay myDisplay, RLKeybindController myKeybinder, string Prompt)
        {
            myKeybinder.HandlingKeys = false;

            RLInput RLI = new RLInput(myDisplay, Prompt);

            RLI.X = (myDisplay.Width - RLI.Width) / 2;
            RLI.Y = (myDisplay.Height - RLI.Height) / 2;

            myDisplay.PopMenu(RLI);

            RLI.ShowTextBox();

            while (myDisplay.ActiveMenu == RLI)
                Application.DoEvents();

            myKeybinder.HandlingKeys = true;

            return RLI.Value();
        }

        public override int Width
        {
            get
            {
                return base.Width;
            }
            set
            {
                base.Width = value;
                FixTextBox();
            }
        }

        public override int Height
        {
            get
            {
                return base.Height;
            }
            set
            {
                base.Height = value;
                FixTextBox();
            }
        }

        public override int X
        {
            get
            {
                return base.X;
            }
            set
            {
                base.X = value;
                FixTextBox();
            }
        }

        public override int Y
        {
            get
            {
                return base.Y;
            }
            set
            {
                base.Y = value;
                FixTextBox();
            }
        }

        private void FixTextBox()
        {
            TB.Top = Y + (4 * CellH);
            TB.Left = Convert.ToInt32(X + (2.5F * CellW));
            TB.Width = Width - (4 * CellW);
        }

        private void ShowTextBox()
        {
            TB.ContextMenu = new ContextMenu();
            TB.BackColor = Color.Black;
            TB.Font = Font;
            TB.BorderStyle = BorderStyle.None;
            TB.ForeColor = Color.White;
            parentDisplay.Controls.Add(TB);
            TB.LostFocus += new EventHandler(TB_LostFocus);
            TB.KeyDown += new KeyEventHandler(TB_KeyDown);
            TB.Focus();
        }

        private void TB_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
            {
                CloseMenu();
            }
            else if (e.KeyCode == Keys.Escape)
            {
                TB.Text = "";
                CloseMenu();
            }
        }      

        private void TB_LostFocus(object sender, EventArgs e)
        {
            TB.Focus();
        }

        protected string myPrompt;
        private TextBox TB = new TextBox();
        private RLDisplay parentDisplay;

        private string myText = "";

        internal string Value()
        {
            return myText;
        }

        internal RLInput(RLDisplay RLD, string iS)
            : base(0, 0, 0, 0)
        {
            Width = (iS.Length + 4) * CellW;
            Height = 7 * CellH;

            parentDisplay = RLD;

            myPrompt = iS;

            DrawMe();
        }

        protected virtual void DrawMe()
        {
            for (int i = 0; i < CellsWide; i++)
            {
                SetCell(i, 0, '%', Color.White, false);
                SetCell(i, CellsTall - 1, '%', Color.LightGray, false);
            }
            for (int i = 0; i < CellsTall; i++)
            {
                SetCell(0, i, '%', Color.White, false);
                SetCell(CellsWide - 1, i, '%', Color.LightGray, false);
            }

            DrawString(myPrompt, 2, 2, Color.White, Color.Black, true);
        }

        protected override void CloseMenu()
        {
            parentDisplay.Controls.Remove(TB);
            myText = TB.Text;
            TB.Dispose();
            GC.Collect();
            parentDisplay.Parent.Focus();
            base.CloseMenu();
        }
    }
}
