using System;
using System.Collections.Generic;
using System.Text;

namespace Phasma
{
    /// <summary>
    /// Abstract base class for all menu objects to be popped up on an RLDisplay.
    /// </summary>
    public abstract class RLMenu : RLWindow
    {
        /// <summary>
        /// Basic constructor for RLMenu Objects.
        /// </summary>
        /// <param name="iX">Starting x-coordinate, in pixels.</param>
        /// <param name="iY">Starting y-coordinate, in pixels.</param>
        /// <param name="iW">Starting width, in pixels.</param>
        /// <param name="iH">Starting height, in pixels.</param>
        public RLMenu(int iX, int iY, int iW, int iH)
            : base(iX, iY, iW, iH, -1, -1)
        {

        }

        internal RLMenu PreviousMenu = null;

        /// <summary>
        /// Causes the menu to handle a given command string.
        /// This method should be overridden in descendant menus to allow them to interact with the user.
        /// If this string is "Close", it will, by default, close the menu.
        /// </summary>
        /// <param name="inCmd"></param>
        public virtual void HandleMenuCommand(string inCmd)
        {
            switch (inCmd)
            {
                case "Close":
                    CloseMenu();
                    break;
            }
        }

        /// <summary>
        /// Closes the menu, removes it from the RLDisplay it was associated with, and restores the previous active menu.
        /// Called by HandleMenuCommand when the given command is "Close".
        /// </summary>
        protected virtual void CloseMenu()
        {
            myParent.ActiveMenu = PreviousMenu;
            myParent.RemWindow(this);
        }
    }
}
