﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Phasma
{
    public class RLMinimap : RLWindow
    {
        /// <summary>
        /// Basic constructor for RLMinimap Objects.
        /// </summary>
        /// <param name="iX">Starting x-coordinate, in pixels.</param>
        /// <param name="iY">Starting y-coordinate, in pixels.</param>
        /// <param name="iW">Starting width, in pixels.</param>
        /// <param name="iH">Starting height, in pixels.</param>
        public RLMinimap(int iX, int iY, int iW, int iH)
            : base(iX, iY, iW, iH, -1, -1)
        {
        }

        /// <summary>
        /// Gets or sets the width of this window's cells, in pixels.
        /// </summary>
        public override int CellW
        {
            get
            {
                return myCellW;
            }
            set
            {
                myCellW = value;
                FlushWindow();
            }
        }

        /// <summary>
        /// Gets or sets the height of this window's cells, in pixels.
        /// </summary>
        public override int CellH
        {
            get
            {
                return myCellH;
            }
            set
            {
                myCellH = value;
                FlushWindow();
            }
        }

        private int myCellOffsetX = 0;
        public int CellOffsetX
        {
            get
            {
                return myCellOffsetX;
            }
            set
            {
                myCellOffsetX = value;
                FlushWindow();
            }
        }

        private int myCellOffsetY = 0;
        public int CellOffsetY
        {
            get
            {
                return myCellOffsetY;
            }
            set
            {
                myCellOffsetY = value;
                FlushWindow();
            }
        }

        internal override void RenderCachedImage()
        {
            Image img = new Bitmap(Width, Height, System.Drawing.Imaging.PixelFormat.Format16bppRgb565);
            Graphics myG = Graphics.FromImage(img);
            myG.FillRectangle(Brushes.Black, 0, 0, Width, Height);
            Rectangle myBounds = Bounds;
            int cW = CellsWide;
            int cT = CellsTall;
            List<Point> dumpList = new List<Point>();
            foreach (KeyValuePair<Point, cellDef> K in myDisplay)
            {
                cellDef CD = K.Value;
                if (CD.X > cW)
                    dumpList.Add(K.Key);
                else if (CD.Y > cT)
                    dumpList.Add(K.Key);
                else
                    myG.FillRectangle(new SolidBrush(CD.TopColor()), (CD.X + CellOffsetX) * CellW, (CD.Y + CellOffsetY) * CellH, CellW, CellH);
            }
            foreach (Point P in dumpList)
                myDisplay.Remove(P);

            DrawFrameRect(myG, 0, 0);

            myCachedImage = img;
        }

        internal override void DrawCell(cellDef CD, bool updateNow)
        {
            if (myParent == null)
                return;
            if (myCachedImage == null)
                RenderCachedImage();

            Graphics cachedGraphics = Graphics.FromImage(myCachedImage);
            cachedGraphics.FillRectangle(new SolidBrush(CD.TopColor()), (CD.X + CellOffsetX) * CellW, (CD.Y + CellOffsetY) * CellH, CellW, CellH);
            DrawFrameRect(cachedGraphics, 0, 0);
            if (updateNow && !myParent.drawingLocked)
            {
                myParent.MyGraphics.FillRectangle(new SolidBrush(CD.TopColor()), X + (CD.X + CellOffsetX) * CellW, Y + (CD.Y + CellOffsetY) * CellH, CellW, CellH);
                DrawFrameRect(myParent.MyGraphics, X, Y);
            }
        }

        private void DrawFrameRect(Graphics myG, int xOffset, int yOffset)
        {
            if (myDisplay.Count > 0)
                myG.DrawRectangle(Pens.White, xOffset, yOffset, Width - 1, Height - 1);
        }

        public void ClearCells()
        {
            base.ClearCells();
            FlushWindow();
        }
    }
}
