using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Phasma
{
    public class Sight
    {
        public delegate bool CanPassFunc(int X, int Y);

        public static bool CheckLoS(int sourceX, int sourceY, int targX, int targY, CanPassFunc canPass, CanPassFunc inBounds)
        {
            int j1, j2;
            return CheckLoS(sourceX, sourceY, targX, targY, canPass, inBounds, out j1, out j2);
        }

        public static bool CheckLoS(int sourceX, int sourceY, int targX, int targY, CanPassFunc canPass, CanPassFunc inBounds, out int endX, out int endY)
        {
            endX = endY = -1;
            return false;
        }

        private static Point viewer = new Point();
        private static CanPassFunc gCanPass;
        private static CanPassFunc gInBounds;
        private static int gAreaWidth, gAreaHeight;

        const int C_VIS = 1;
        const int C_INVIS = 0;
        const int C_INVIS_CHECKED = -1;

        private static bool[,] rV = null;
        private static int[,] corners = null;

        public static bool[,] GetFoV(int sourceX, int sourceY, int fieldRad, int areaWidth, int areaHeight, CanPassFunc canPass, CanPassFunc inBounds)
        {
            rV = new bool[areaWidth, areaHeight];
            corners = new int[areaWidth + 1, areaHeight + 1];

            int maxRadius = fieldRad;

            viewer.X = sourceX;
            viewer.Y = sourceY;
            gCanPass = canPass;
            gInBounds = inBounds;
            gAreaWidth = areaWidth;
            gAreaHeight = areaHeight;

            Point start = new Point(viewer.X, viewer.Y), end = new Point(0, 0);

            for (int l = 1; l < maxRadius; l++)
            {
                end.Y = start.Y - l;
                if (end.Y >= 0)
                    for (end.X = start.X - l; end.X < start.X + l + 1; end.X++)
                        CheckWayToCorner(start, end, maxRadius);
                end.X = start.X + l + 1;
                if (end.X < gAreaWidth + 1)
                    for (end.Y = start.Y - l; end.Y < start.Y + l + 1; end.Y++)
                        CheckWayToCorner(start, end, maxRadius);
                end.Y = start.Y + l + 1;
                if (end.Y < gAreaHeight + 1)
                    for (end.X = start.X + l + 1; end.X > start.X - l; end.X--)
                        CheckWayToCorner(start, end, maxRadius);
                end.X = start.X - l;
                if (end.X >= 0)
                    for (end.Y = start.Y + l + 1; end.Y > start.Y - l; end.Y--)
                        CheckWayToCorner(start, end, maxRadius);
            }

            ConvertCornersToVisibleCells();
            CorrectNearest(start);

            rV[start.X, start.Y] = true;

            return rV;
        }

        private static void CorrectNearest(Point start)
        {
            int[,] singleCheck = {
                {-1, -1, -2, -2},
                {1, 1, 2, 2},
                {-1, 1, -2, 2},
                {1, -1, 2, -2},
                {-1, 0, -2, 0},
                {1, 0, 2, 0},
                {0, -1, 0, -2},
                {0, 1, 0, 2}};

            for (int index = 0; index < 8; index++)
                if (CantSeeThrough(start.X + singleCheck[index, 0], start.Y + singleCheck[index, 1]))
                    if (gInBounds(start.X + singleCheck[index, 2], start.Y + singleCheck[index, 3]))
                        rV[start.X + singleCheck[index, 2], start.Y + singleCheck[index, 3]] = false;

            int[,] doubleCheck = {
            {-1,-1, 0,-1, -1,-2},
            {1,-1,  0,-1,  1,-2},
            {-1,1,  0,1,  -1,2},
            {1,1,   0,1,   1,2},
		
            {-1,-1,-1,0,  -2,-1},
            {-1,1, -1,0,  -2,1},
            {1,-1,  1,0,   2,-1},
            {1,1,   1,0,   2,1}};

            for (int index = 0; index < 8; index++)
                if (CantSeeThrough(start.X + doubleCheck[index, 0], start.Y + doubleCheck[index, 1])
                    && CantSeeThrough(start.X + doubleCheck[index, 2], start.Y + doubleCheck[index, 3]))
                    if (gInBounds(start.X + doubleCheck[index, 4], start.Y + doubleCheck[index, 5]))
                        rV[start.X + doubleCheck[index, 4], start.Y + doubleCheck[index, 5]] = false;


        }

        private static void ConvertCornersToVisibleCells()
        {
            for (int y = 0; y < gAreaHeight + 1; y++)
                for (int x = 0; x < gAreaWidth + 1; x++)
                    if (corners[x, y] == C_VIS)
                    {
                        if (x < gAreaWidth && y < gAreaHeight) rV[x, y] = true;
                        if (x < gAreaWidth && y > 0) rV[x, y - 1] = true;
                        if (x > 0 && y < gAreaHeight) rV[x - 1, y] = true;
                        if (x > 0 && y > 0) rV[x - 1, y - 1] = true;
                    }
        }

        private static void CheckWayToCorner(Point start, Point end, int distance)
        {
            if (end.X >= 0 && end.X < gAreaWidth && end.Y >= 0 && end.Y <= gAreaHeight)
                if (corners[end.X, end.Y] != C_INVIS_CHECKED)
                    CheckLOS(start, end, distance);
        }

        private static void CheckLOS(Point start, Point end, int distance)
        {
            Point start2 = new Point(0, 0);

            for (int a = 0; a < 2; a++)
                for (int b = 0; b < 2; b++)
                {
                    start2.X = start.X + a;
                    start2.Y = start.Y + b;

                    if (!AnyCellBlocks(start2, end))
                    {
                        corners[end.X, end.Y] = C_VIS;
                        return;
                    }

                }

            Point diff = new Point(end.X - start.X, end.Y - start.Y);
            PropagateShadow(start, diff, distance);

        }

        private static void PropagateShadow(Point start, Point diff, int distance)
        {
            Point newpos = new Point(start.X + diff.X, start.Y + diff.Y);
            while (Math.Abs(start.X - newpos.X) < distance && Math.Abs(start.Y - newpos.Y) < distance)
            {
                if (newpos.X >= 0 && newpos.X <= gAreaWidth && newpos.Y >= 0 && newpos.Y <= gAreaHeight)
                    corners[newpos.X, newpos.Y] = C_INVIS_CHECKED;
                else
                    return;
                newpos.X += diff.X;
                newpos.Y += diff.Y;
            }
        }

        private static bool CantSeeThrough(int X, int Y)
        {
            if (X < 0 || Y < 0 || X >= gAreaWidth || Y >= gAreaHeight)
                return true;

            return !gCanPass(X, Y);

        }

        private static bool AnyCellBlocks(Point start, Point end)
        {
            int s_x = Math.Sign(end.X - start.X), s_y = Math.Sign(end.Y - start.Y);

            if (s_x == 1 && s_y == 1 && CantSeeThrough(start.X, start.Y))
                return true;
            if (s_x == -1 && s_y == 1 && CantSeeThrough(start.X - 1, start.Y))
                return true;
            if (s_x == -1 && s_y == -1 && CantSeeThrough(start.X - 1, start.Y - 1))
                return true;
            if (s_x == 1 && s_y == -1 && CantSeeThrough(start.X, start.Y - 1))
                return true;

            int dx = Math.Abs(end.X - start.X), dy = Math.Abs(end.Y - start.Y);

            int nx, ny, rx, ry, cellx, celly, x, y;

            if (dx != 0)
            {
                for (x = 1; x < dx; x++)
                {
                    ny = x * dy / dx;
                    ry = x * dy % dx;

                    celly = start.Y + ny * s_y;
                    cellx = start.X + x * s_x;
                    if (ry != 0)
                    {
                        if (s_y == -1)
                            celly--;
                        if (CantSeeThrough(cellx - 1, celly) || CantSeeThrough(cellx, celly))
                            return true;
                    }
                    else
                    {
                        if (CornerBlocks(cellx, celly, s_x, s_y))
                            return true;
                    }
                }
            }

            if (dy != 0)
            {
                for (y = 1; y < dy; y++)
                {
                    nx = y * dx / dy;
                    rx = y * dx % dy;

                    cellx = start.X + nx * s_x;
                    celly = start.Y + y * s_y;
                    if (rx != 0)
                    {
                        if (s_x == -1)
                            cellx--;
                        if (CantSeeThrough(cellx, celly - 1) || CantSeeThrough(cellx, celly))
                            return true;
                    }
                    else
                    {
                        if (CornerBlocks(cellx, celly, s_x, s_y))
                            return true;
                    }
                }
            }

            return false;
        }

        private static bool CornerBlocks(int cornerx, int cornery, int s_x, int s_y)
        {
            if (CantSeeThrough(cornerx, cornery) && CantSeeThrough(cornerx, cornery - 1))
                return true;
            if (CantSeeThrough(cornerx, cornery) && CantSeeThrough(cornerx - 1, cornery))
                return true;
            if (CantSeeThrough(cornerx - 1, cornery - 1) && CantSeeThrough(cornerx, cornery - 1))
                return true;
            if (CantSeeThrough(cornerx - 1, cornery - 1) && CantSeeThrough(cornerx - 1, cornery))
                return true;

            if (s_x * s_y == 1)
            {
                if (CantSeeThrough(cornerx, cornery) || CantSeeThrough(cornerx - 1, cornery - 1))
                    return true;
            }
            else if (s_x * s_y == -1)
            {
                if (CantSeeThrough(cornerx, cornery - 1) || CantSeeThrough(cornerx - 1, cornery))
                    return true;
            }
            else
            {
                if (CantSeeThrough(cornerx - 1, cornery - 1) || CantSeeThrough(cornerx - 1, cornery) ||
                    CantSeeThrough(cornerx, cornery - 1) || CantSeeThrough(cornerx, cornery))
                    return true;

            }
            return false;

        }
    }
}
