using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace PhasmaDemoApp
{
    public class menuMain : Phasma.RLMenu
    {
        private int myWidth = 25; //in cells, not pixels
        private int myHeight = 13; //in cells, not pixels

        //this window's position will be adjusted by its own code, 
        //so we'll ignore the base object's ctor arguments
        public menuMain(int pWidth, int pHeight)
            : base(0, 0, 1, 1)
        {
            //Set the application's state to "Menu"-
            //this causes the keybind handler stuff to route commands here
            Form1.MyState = "Menu";
            //Attach a handler for mouse clicks
            base.MouseClick += new MouseClickHandler(menuMain_MouseClick);

            //center the window
            Width = CellW * myWidth;
            Height = CellH * myHeight;
            X = (pWidth - (myWidth * CellW)) / 2;
            Y = (pHeight - (myHeight * CellH)) / 2;

            //draw a fancy border on the window
            for (int x = 1; x < CellsWide - 1; x++)
            {
                SetCell(x, 0, '-', Color.Red, false);
                SetCell(x, CellsTall - 1, '-', Color.Red, false);
            }
            for (int y = 1; y < CellsTall - 1; y++)
            {
                SetCell(0, y, '|', Color.Red, false);
                SetCell(CellsWide - 1, y, '|', Color.Red, false);
            }

            SetCell(0, 0, '/', Color.Red, false);
            SetCell(CellsWide - 1, 0, '\\', Color.Red, false);
            SetCell(0, CellsTall - 1, '\\', Color.Red, false);
            SetCell(CellsWide - 1, CellsTall - 1, '/', Color.Red, false);

            //Draw it
            RenderMenu();

            Update();
        }

        //option the user currently has selected
        private int SelectedOption = 0;

        //helper function to draw a string centered in the window
        private void DrawCenteredString(string s, int y, bool selected)
        {
            DrawString(s, (CellsWide - s.Length) / 2, y, Color.Red, selected ? Color.DarkRed : Color.Black, false);
        }

        //Draws the main menu options
        private void RenderMenu()
        {
            DrawCenteredString("Main Menu", 1, false);

            DrawCenteredString("Toggle Stats", 3, SelectedOption == 0);
            DrawCenteredString("Toggle Messages", 4, SelectedOption == 1);
            DrawCenteredString("Toggle Map", 5, SelectedOption == 2);
            DrawCenteredString("Show Text Input", 6, SelectedOption == 3);
            DrawCenteredString("Toggle Images", 7, SelectedOption == 4);
            DrawCenteredString("Toggle Commands", 8, SelectedOption == 5);

            DrawCenteredString("Images by David Gervais", 10, false);

            Update();
        }

        //Set the application's state back to "Normal"
        protected override void CloseMenu()
        {
            base.CloseMenu();
            Form1.MyState = "Normal";
        }

        //Figure out the option they've clicked on, and handle it
        void menuMain_MouseClick(int cellX, int cellY, System.Windows.Forms.MouseButtons Buttons)
        {
            if (Buttons == System.Windows.Forms.MouseButtons.Left)
            {
                int newSelection = cellY - 3;
                if (newSelection >= 0 || newSelection <= 5)
                {
                    SelectedOption = newSelection;
                    RenderMenu();
                    HandleMenuCommand("OK");
                }
            }
        }

        public override void HandleMenuCommand(string inCmd)
        {            
            if (inCmd == "Up")
            {
                if (SelectedOption > 0)
                {
                    SelectedOption--;
                    RenderMenu();
                }
            }
            else if (inCmd == "Down")
            {
                if (SelectedOption < 5)
                {
                    SelectedOption++;
                    RenderMenu();
                }
            }
            else if (inCmd == "OK")
            {
                switch (SelectedOption)
                {
                    case 0:
                        Form1.ToggleStats();
                        break;
                    case 1:
                        Form1.ToggleMessages();
                        break;
                    case 2:
                        Form1.ToggleMap();
                        break;
                    case 3:
                        Form1.ShowInput();
                        break;
                    case 4:
                        Form1.ToggleImages();
                        break;
                    case 5:
                        Form1.ToggleCommands();
                        break;
                }
            }
            else
                base.HandleMenuCommand(inCmd);
        }
    }
}
